@@ -1,6 +1,10 @@
-#' Display the raw periodogram for a time series, not on a log scale.
-#' The standard "R" functions display periodograms on a log scale which can make it more difficult to locate high peaks in the spectrum at differning frequencies.
+#' Display raw periodogram
+#'
+#' Display the raw periodogram for a time series, and not on a log scale.
+#'
+#' The standard "R" functions display periodograms on a log scale which can make it more difficult to locate high peaks in the spectrum at differing frequencies.
 #' This routine will display the peaks on a raw scale.
+#'
 #' @param x (num) This should be a numeric vector representing the process to estimate.
 #' @param k (int) The number of Gegenbauer factors
 #' @return A ggplot object representing the raw periodogram
@@ -28,6 +32,7 @@ gg_raw_pgram <- function(x,k=1) {
   # now plot it
   ggplot2::ggplot(data=df,ggplot2::aes(x=Frequency,y=Intensity)) +
     ggplot2::geom_line() +
+    ggplot2::ggtitle(paste('Raw periogram of',deparse(match.call()$x))) +
     ggplot2::geom_text(data=annotate_df,aes(x=x,y=y,label=label),size=2.5,hjust=0) +
     ggplot2::theme_bw()
 }
